
/*
   Water Meter Project
   Kevin Darrah

   Twitter: https://twitter.com/KDcircuits
   For inquiries or design services:
   https://www.kdcircuits.com

   USED AN ESP32 for this - called this on Amazon "HiLetgo ESP-WROOM-32 ESP32 ESP-32S Development Board"
   Tools set to NodeMCU-32S

   Hall Effect Sensor: DRV5056A1ELPGMQ1

   Tutorial I found that inspired this: https://www.instructables.com/id/Monitoring-residential-water-usage-by-reading-muni/

*/

//uncomment this line for testing the magnetic wheel, run the serial plotter
//#define waterFlowTestMode


#include "config.h"
unsigned long pulseCount = 0;

int rollingArray[300];
float runningAverage, shortRunningAverage;
float gallonsUsed;
const float conversionToGalConstant = 0.03, pulseThreshold = 3.2;
const int hallEffectPin = 34;
boolean highPulseActive = false;
long lastPulseTime;
boolean firstBoot = true;
long lastTimeSent;

AdafruitIO_Feed *gallons = io.feed("gallons");


void setup() {


  Serial.begin(115200);
  while (! Serial);

#ifndef waterFlowTestMode
  Serial.println("Connecting to Adafruit IO");
  // connect to io.adafruit.com
  io.connect();
  gallons->onMessage(handleMessage);
  // wait for a connection
  int connectionTryCount = 0;
  while (io.status() < AIO_CONNECTED) {
    Serial.print(".");
    delay(1000);
    connectionTryCount++;
    if (connectionTryCount > 10)//if we can't connect, reboot and try again
      ESP.restart();
  }

  // we are connected
  Serial.println();
  Serial.println(io.statusText());

  gallons->get();// get latest value
  for (int i = 0; i < 500; i++) {// wait until that value is updated before continuing
    io.run();//check
    delay(10);
    if (!firstBoot)
      break;
  }
  if (firstBoot)
    ESP.restart();//never updated, so reboot and try again
#endif

  lastTimeSent = millis();
  lastPulseTime = millis();
}

void loop() {
  //****************************************  send up to cloud
#ifndef waterFlowTestMode
  if (millis() - lastTimeSent > 60000) {
    if (io.status() != AIO_CONNECTED) {//reboot if connection lost
      ESP.restart();
    }

    lastTimeSent = millis();
    io.run();
    Serial.print("Sending Data ");
    gallons->save(gallonsUsed);
    Serial.print(" ");
    Serial.print(millis() - lastTimeSent);
    Serial.println("ms to do this ");
  }
#endif
  //****************************************
  //****************************************
  //****************************************
  //****************************************
  //****************************************  rolling averages
  runningAverage = 0;
  shortRunningAverage = 0;
  for (int i = 0; i < 299; i++) {
    rollingArray[i] = rollingArray[i + 1];
    runningAverage = runningAverage + rollingArray[i];
  }
  for (int i = 270; i < 299;  i++) {
    shortRunningAverage = shortRunningAverage + rollingArray[i];
  }
  rollingArray[299] = analogRead(hallEffectPin);
  runningAverage = runningAverage + rollingArray[299];
  shortRunningAverage = shortRunningAverage + rollingArray[299];
  shortRunningAverage = shortRunningAverage / 30.000;
  runningAverage = runningAverage / 300.000;
  //****************************************
  //****************************************
  //****************************************
  //****************************************
  //****************************************  detection
  float flatValue  =  shortRunningAverage - runningAverage;
  if (flatValue > pulseThreshold && highPulseActive == false) {
    highPulseActive = true;
#ifndef waterFlowTestMode
    pulseCount++;
    Serial.print("Pulses: ");
    Serial.print(pulseCount);
    Serial.print("  Gallons:  ");
    gallonsUsed = conversionToGalConstant * pulseCount;
    Serial.print(gallonsUsed);
    Serial.print("  Time Since Last");
    Serial.println(millis() - lastPulseTime);
    lastPulseTime = millis();
#endif
  }
  else if (flatValue < (-1 * pulseThreshold) && highPulseActive == true) {
    highPulseActive = false;
  }
#ifdef waterFlowTestMode
  Serial.print(flatValue);
  Serial.print(",");
  if (highPulseActive)
    Serial.println(pulseThreshold);
  else
    Serial.println(0);
#endif


  delay(10);
}

void handleMessage(AdafruitIO_Data *data) {
  if (firstBoot) {
    firstBoot = false;
    Serial.print("received <-  ");
    Serial.println(data->toFloat());
    float gallonsFromCloud = data->toFloat();
    pulseCount = gallonsFromCloud / conversionToGalConstant;
    Serial.print("Pulse Count Set to: ");
    Serial.println(pulseCount);
  }
}
